@@ -1,6 +1,6 @@
 /* contentdir.c - Implementation of UPnP ContentDirectory
  *
- * Copyright (C) 2005  Oskar Liljeblad
+ * Copyright (C) 2005, 2006  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <config.h>
 #include <stdint.h>		/* Gnulib/C99 */
 #include <inttypes.h>		/* ? */
+#include <sys/stat.h>		/* POSIX */
 #include "gettext.h"		/* Gnulib/gettext */
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
@@ -29,9 +30,11 @@
 #include "xalloc.h"		/* Gnulib */
 #include "quotearg.h"		/* Gnulib */
 #include "dirname.h"		/* Gnulib */
+#include "c-ctype.h"		/* Gnulib */
 #include "strbuf.h"
 #include "intutil.h"
 #include "strutil.h"
+#include "tmap.h"
 #include "gmediaserver.h"
 #include "search-parser.h"
 #include "search-lexer.h"
@@ -63,6 +66,112 @@ bump_update_id(void)
     notify_change(CONTENTDIR_SERVICE_ID, var);
 }
 
+static char *
+get_entry_property(const Entry *entry, char *property)
+{
+    EntryDetail *detail;
+
+    detail = get_entry_detail(entry, DETAIL_TAG);
+
+    /* XXX: should we allow namespace-less property names,
+     * such as "title" rather than "dc:title"?
+     * XXX: should we allow attribute properties,
+     * such as @refID, @id, @parentID or @protocolInfo?
+     */
+
+    /* See the ContentDirectory spec for an example where @id is used. */
+    if (strcmp(property, "id") == 0 || strcmp(property, "@id") == 0) {
+        char *t1, *t2;
+        t1 = xasprintf("%" PRId32, entry->id); /* " */
+        t2 = convert_string_to_device(t1);
+        free(t1);
+        return t2;
+    }
+
+    if (strcmp(property, "dc:title") == 0) {
+	if (detail != NULL && detail->data.tag.title != NULL && strcmp(detail->data.tag.title, "") != 0)
+	    return convert_string_to_device(detail->data.tag.title);
+        return convert_string_to_device(entry->name);
+    }
+
+    if (strcmp(property, "upnp:class") == 0) {
+	if (has_entry_detail(entry, DETAIL_CHILDREN))
+	    return xstrdup("object.container.storageFolder");
+        /*if (has_entry_detail(entry, DETAIL_URL))
+            return xstrdup("object.item.audioItem.audioBroadcast");*/
+	return xstrdup("object.item.audioItem.musicTrack");
+    }
+
+    if (detail != NULL) {
+        /* XXX: Accept dc:creator as a synonym to upnp:artist? */
+	if (strcmp(property, "upnp:artist") == 0 && detail->data.tag.artist != NULL)
+	    return convert_string_to_device(detail->data.tag.artist);
+	if (strcmp(property, "upnp:album") == 0 && detail->data.tag.album != NULL)
+	    return convert_string_to_device(detail->data.tag.album);
+	if (strcmp(property, "upnp:genre") == 0 && detail->data.tag.genre != NULL)
+	    return convert_string_to_device(detail->data.tag.genre);
+        if (strcmp(property, "upnp:originalTrackNumber") == 0 && detail->data.tag.track_no != -1) {
+            char *t1, *t2;
+            t1 = xasprintf("%" PRId32, detail->data.tag.track_no);
+            t2 = convert_string_to_device(t1);
+            free(t1);
+            return t2;
+        }
+    }
+
+    return NULL;
+}
+
+int
+sort_criteria_comparision(const void *k1, const void *k2, void *data)
+{
+    SortCriteria *criteria = data;
+    SortCriteriaEntry *sort_entry;
+    const Entry *e1 = k1;
+    const Entry *e2 = k2;
+
+    sort_entry = criteria->first;
+    while (sort_entry != NULL) {
+	char *p1 = get_entry_property(e1, sort_entry->property);
+	char *p2 = get_entry_property(e2, sort_entry->property);
+
+	if (p1 == NULL && p2 != NULL) {
+	    free(p2);
+	    return (sort_entry->ascending ? -1 : 1);
+	}
+	if (p1 != NULL && p2 == NULL) {
+	    free(p1);
+	    return (sort_entry->ascending ? 1 : -1);
+	}
+	if (p1 != NULL && p2 != NULL) {
+	    int compare;
+
+	    compare = strcmp(p1, p2);
+	    free(p1);
+	    free(p2);
+	    if (compare != 0)
+		return (sort_entry->ascending ? compare : -compare);
+	}
+    }
+
+    return e1-e2;
+}
+
+void
+free_sort_criteria(SortCriteria *criteria)
+{
+    SortCriteriaEntry *entry;
+
+    for (entry = criteria->first; entry != NULL; ) {
+	SortCriteriaEntry *next = entry->next;
+	free(entry->property);
+	free(entry);
+	entry = next;
+    }
+
+    free(criteria);
+}
+
 void
 free_search_criteria(SearchCriteria *criteria)
 {
@@ -96,14 +205,14 @@ free_search_expr(SearchExpr *expr)
 static bool
 contentdir_get_search_capabilities(ActionEvent *event)
 {
-    upnp_add_response(event, "SearchCaps", "");
+    upnp_add_response(event, "SearchCaps", "*");
     return event->status;
 }
 
 static bool
 contentdir_get_sort_capabilities(ActionEvent *event)
 {
-    upnp_add_response(event, "SortCaps", "");
+    upnp_add_response(event, "SortCaps", "*");
     return event->status;
 }
 
@@ -124,68 +233,44 @@ append_escaped_xml(StrBuf *out, const char *entity, const char *value)
     free(str);
 }
 
-void
-end_result(StrBuf *result)
+TMap *
+new_result_set(SortCriteria *criteria)
 {
-    strbuf_append(result, "</DIDL-Lite>");
+    TMap *results;
+
+    results = tmap_new();
+    tmap_set_complex_compare_fn(results, sort_criteria_comparision, criteria);
+
+    return results;
 }
 
 void
-begin_result(StrBuf *result)
+free_result_set(TMap *results)
 {
-    strbuf_append(result, 
+    tmap_foreach_value(results, free);
+    tmap_free(results);
+}
+
+char *
+result_set_to_string(TMap *results)
+{
+    StrBuf *out;
+    TMapIterator it;
+    
+    out = strbuf_new();
+    strbuf_append(out, 
             "<DIDL-Lite xmlns=\"urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/\""
             " xmlns:dc=\"http://purl.org/dc/elements/1.1/\""
             " xmlns:upnp=\"urn:schemas-upnp-org:metadata-1-0/upnp/\">");
-}
 
-static char *
-get_entry_property(Entry *entry, char *property)
-{
-    EntryDetail *detail;
-
-    detail = get_entry_detail(entry, DETAIL_TAG);
-
-    /* XXX: should we allow namespace-less property names,
-     * such as "title" rather than "dc:title"?
-     * XXX: should we allow attribute properties,
-     * such as @refID, @id, @parentID or @protocolInfo?
-     */
-
-    /* See the ContentDirectory spec for an example where @id is used. */
-    if (strcmp(property, "id") == 0 || strcmp(property, "@id") == 0) {
-        char *t1, *t2;
-        t1 = xasprintf("%" PRId32, entry->id); /* " */
-        t2 = convert_string(t1);
-        free(t1);
-        return t2;
+    tmap_iterator(results, &it);
+    while (it.has_next(&it)) {
+	char *entry = it.next(&it);
+	strbuf_append(out, entry);
     }
 
-    if (strcmp(property, "dc:title") == 0) {
-	if (detail != NULL && detail->data.tag.title != NULL && strcmp(detail->data.tag.title, "") != 0)
-	    return convert_string(detail->data.tag.title);
-        return convert_string(entry->name);
-    }
-
-    if (strcmp(property, "upnp:class") == 0) {
-	if (has_entry_detail(entry, DETAIL_CHILDREN))
-	    return convert_string("object.container.storageFolder");
-        /*if (has_entry_detail(entry, DETAIL_URL))
-            return convert_string("object.item.audioItem.audioBroadcast");*/
-	return convert_string("object.item.audioItem.musicTrack");
-    }
-
-    if (detail != NULL) {
-        /* XXX: Accept dc:creator as a synonym to upnp:artist? */
-	if (strcmp(property, "upnp:artist") == 0 && detail->data.tag.artist != NULL)
-	    return convert_string(detail->data.tag.artist);
-	if (strcmp(property, "upnp:album") == 0 && detail->data.tag.album != NULL)
-	    return convert_string(detail->data.tag.album);
-	if (strcmp(property, "upnp:genre") == 0 && detail->data.tag.genre != NULL)
-	    return convert_string(detail->data.tag.genre);
-    }
-
-    return NULL;
+    strbuf_append(out, "</DIDL-Lite>");
+    return strbuf_free_to_string(out);
 }
 
 static bool
@@ -226,11 +311,13 @@ filter_includes(const char *filter, const char *field)
 }
 
 static void
-append_entry(StrBuf *result, Entry *entry, const char *filter)
+add_result_entry(TMap *results, Entry *entry, const char *filter)
 {
     EntryDetail *detail;
     char *value;
+    StrBuf *result;
 
+    result = strbuf_new();
     if (has_entry_detail(entry, DETAIL_CHILDREN)) {
         detail = get_entry_detail(entry, DETAIL_CHILDREN);
 	strbuf_appendf(result,
@@ -247,11 +334,31 @@ append_entry(StrBuf *result, Entry *entry, const char *filter)
             detail = get_entry_detail(entry, DETAIL_FILE);
             if (detail != NULL) {
                 strbuf_append(result, "<res");
+
                 if (filter_includes_any(filter, "res", "@protocolInfo", "res@protocolInfo", NULL))
                     strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:*\"", detail->data.file.mime_type);
-                /*if (detail->data.file.size != 0)*/
-                if (filter_includes_any(filter, "res", "@size", "res@size", NULL))
-                    strbuf_appendf(result, " size=\"%" PRIu32 "\"", detail->data.file.size);
+
+		if (S_ISREG(detail->data.file.mode)) { /*if (detail->data.file.size != 0)*/
+	            if (filter_includes_any(filter, "res", "@size", "res@size", NULL))
+                        strbuf_appendf(result, " size=\"%" PRIu32 "\"", detail->data.file.size);
+
+                    if (has_entry_detail(entry, DETAIL_TAG)) {
+                        EntryDetail *tag_detail = get_entry_detail(entry, DETAIL_TAG);
+                        if (filter_includes_any(filter, "res", "@duration", "res@duration", NULL))
+                            strbuf_appendf(result, " duration=\"%u:%02u:%02u.%02u\"", SPLIT_DURATION(tag_detail->data.tag.duration));
+                    }
+#if 0
+                    if (filter_includes_any(filter, "res", "@bitrate", "res@bitrate", NULL))
+                        strbuf_appendf(result, " bitrate=\"%u\"", 0);
+                    
+                    if (filter_includes_any(filter, "res", "@bitsPerSample", "res@bitsPerSample", NULL))
+                        strbuf_appendf(result, " bitsPerSample=\"%u\"", 0);
+                    
+                    if (filter_includes_any(filter, "res", "@sampleFrequency", "res@sampleFrequency", NULL))
+                        strbuf_appendf(result, " sampleFrequency=\"%u\"", 0);
+#endif
+                }
+
                 strbuf_appendf(result, ">http://%s:%d/files/%d</res>",
                         UpnpGetServerIpAddress(),
                         UpnpGetServerPort(),
@@ -298,12 +405,20 @@ append_entry(StrBuf *result, Entry *entry, const char *filter)
 	    append_escaped_xml(result, "upnp:genre", value);
         free(value);
     }
+    if (filter_includes(filter, "upnp:originalTrackNumber")) {
+        value = get_entry_property(entry, "upnp:originalTrackNumber");
+        if (value != NULL)
+            append_escaped_xml(result, "upnp:originalTrackNumber", value);
+        free(value);
+    }
 
     if (has_entry_detail(entry, DETAIL_CHILDREN)) {
 	strbuf_append(result, "</container>");
     } else {
 	strbuf_append(result, "</item>");
     }
+
+    tmap_put(results, entry, strbuf_free_to_string(result));
 }
 
 static char *
@@ -327,6 +442,21 @@ operator_name(int type)
     return NULL;
 }
 
+static void
+dump_sort_criteria(SortCriteria *criteria)
+{
+    SortCriteriaEntry *entry;
+
+    say(4, _("Sort criteria:\n"));
+    for (entry = criteria->first; entry != NULL; entry = entry->next) {
+	say(4, "  %s (%s)\n",
+		entry->property,
+		entry->ascending ? _("ascending") : _("descending"));
+    }
+    if (criteria->first == NULL)
+	say(4, _("  (empty)\n"));
+}
+
 static void
 dump_search_expr(SearchExpr *e, int indent_size)
 {
@@ -335,27 +465,72 @@ dump_search_expr(SearchExpr *e, int indent_size)
     indent = xstrdupn(" ", indent_size);
     if (e == NULL) {
 	say(4, _("%sMatch anything\n"), indent);
-	free(indent);
-	return;
-    }
-
-    if (e->type == T_AND || e->type == T_OR) {
-	say(4, _("%s%s operator\n"), indent, operator_name(e->type));
-	say(4, _("%sExpression 1:\n"), indent);
-	dump_search_expr(e->u.logical.expr1, indent_size+2);
-	say(4, _("%sExpression 2:\n"), indent);
-	dump_search_expr(e->u.logical.expr2, indent_size+2);
-    } else if (e->type == T_EXISTS) {
-	say(4, _("%s%s operator\n"), indent, operator_name(e->type));
-	say(4, _("%sProperty: %s\n"), indent, e->u.exists.property);
-	say(4, _("%sExistence: %s\n"), indent, e->u.exists.must_exist ? _("true") : _("false"));
     } else {
 	say(4, _("%s%s operator\n"), indent, operator_name(e->type));
-	say(4, _("%sProperty: %s\n"), indent, e->u.binary.property);
-	say(4, _("%sValue: %s\n"), indent, e->u.binary.value);
+	if (e->type == T_AND || e->type == T_OR) {
+	    say(4, _("%sExpression 1:\n"), indent);
+	    dump_search_expr(e->u.logical.expr1, indent_size+2);
+	    say(4, _("%sExpression 2:\n"), indent);
+	    dump_search_expr(e->u.logical.expr2, indent_size+2);
+	} else if (e->type == T_EXISTS) {
+	    say(4, _("%sProperty: %s\n"), indent, e->u.exists.property);
+	    say(4, _("%sExistence: %s\n"), indent, e->u.exists.must_exist ? _("true") : _("false"));
+	} else {
+	    say(4, _("%sProperty: %s\n"), indent, e->u.binary.property);
+	    say(4, _("%sValue: %s\n"), indent, e->u.binary.value);
+	}
+    }
+    free(indent);
+}
+
+static SortCriteria *
+parse_sort_criteria(const char *str, const char **error)
+{
+    SortCriteria *criteria;
+    SortCriteriaEntry *previous_entry = NULL;
+
+    criteria = xmalloc(sizeof(SortCriteria));
+    criteria->first = NULL;
+
+    for (;;) {
+	SortCriteriaEntry *entry;
+	bool ascending;
+	const char *end;
+
+	for (; c_isspace(*str); str++);
+	if (*str == '\0')
+	    break;
+	if (*str == '+') {
+	    ascending = true;
+	} else if (*str == '-') {
+	    ascending = false;
+	} else {
+	    free_sort_criteria(criteria);
+	    *error = _("invalid sort sign (should be `+' or `-')");
+	    return NULL;
+	}
+	str++;
+	for (end = str; *end != '\0' && *end != ',' && !c_isspace(*end); end++);
+	if (str == end) {
+	    free_sort_criteria(criteria);
+	    *error = _("missing property name");
+	    return NULL;
+	}
+	entry = xmalloc(sizeof(SortCriteriaEntry));
+	entry->property = xmemdup(str, end-str);
+	entry->ascending = ascending;
+	entry->next = NULL;
+	if (criteria->first == NULL)
+	    criteria->first = entry;
+	else
+	    previous_entry->next = entry;
+	previous_entry = entry;
+	str = end;
+	if (*str == ',')
+	    str++;
     }
 
-    free(indent);
+    return criteria;
 }
 
 static SearchCriteria *
@@ -477,7 +652,7 @@ match_search_criteria(SearchCriteria *criteria, Entry *entry)
 static uint32_t
 search_container(SearchCriteria *criteria, Entry *entry,
 		 uint32_t *skip, uint32_t max_count,
-		 const char *filter, StrBuf *result)
+		 const char *filter, TMap *results)
 {
     EntryDetail *detail;
     uint32_t c;
@@ -495,7 +670,7 @@ search_container(SearchCriteria *criteria, Entry *entry,
 		(*skip)--;
 	    } else {
 	        /*if (result != NULL)*/
-                append_entry(result, child, filter);
+                add_result_entry(results, child, filter);
 		match_count++;
 		if (match_count > max_count)
 		    return match_count;
@@ -508,7 +683,7 @@ search_container(SearchCriteria *criteria, Entry *entry,
 	     * can return more than UINT32_MAX results anyway
 	     * (the return value would overflow).
 	     */
-	    match_count += search_container(criteria, child, skip, max_count-match_count, filter, result);
+	    match_count += search_container(criteria, child, skip, max_count-match_count, filter, results);
 	    if (match_count > max_count)
 		return match_count;
 	}
@@ -524,19 +699,21 @@ contentdir_search(ActionEvent *event)
     uint32_t index;
     uint32_t count;
     uint32_t match_count;
-    char *search_criteria;
-    char *sort_criteria;
+    char *search_criteria_str;
+    char *sort_criteria_str;
     char *filter;
     const char *error = NULL;
     Entry *entry;
     EntryDetail *detail;
-    SearchCriteria *criteria;
-    StrBuf *result;
+    SearchCriteria *search_criteria;
+    SortCriteria *sort_criteria;
+    TMap *results;
+    char *results_str;
 
     /* Retrieve arguments */
     id = upnp_get_i4(event, "ContainerID"); /* See comment for ObjectID in browse */
-    search_criteria = upnp_get_string(event, "SearchCriteria");
-    sort_criteria = upnp_get_string(event, "SortCriteria");
+    search_criteria_str = upnp_get_string(event, "SearchCriteria");
+    sort_criteria_str = upnp_get_string(event, "SortCriteria");
     filter = upnp_get_string(event, "Filter");
     strreplace(filter, ' ', ','); /* XXX: This is a hack. Better remove all spaces. */
     index = upnp_get_ui4(event, "StartingIndex");
@@ -548,23 +725,27 @@ contentdir_search(ActionEvent *event)
     if (count == 0)
         count = UINT32_MAX;
 
-    /* SortCriteria not supported at the moment */
-    if (strcmp(sort_criteria, "") != 0) {
+    /* Check SortCriteria. */
+    sort_criteria = parse_sort_criteria(sort_criteria_str, &error);
+    if (sort_criteria == NULL) {
         upnp_set_error(event, UPNP_CONTENTDIR_E_BAD_SORT_CRITERIA,
-            _("Sorting not supported"));
-        return false;
+            _("Invalid sort criteria: %s"), error);
+	return false;
     }
+    if (verbosity >= 4)
+	dump_sort_criteria(sort_criteria);
 
-    /* Check SearchCriteria */
-    criteria = parse_search_criteria(search_criteria, &error);
-    if (criteria == NULL) {
-        upnp_set_error(event, UPNP_CONTENTDIR_E_BAD_SORT_CRITERIA,
+    /* Check SearchCriteria. */
+    search_criteria = parse_search_criteria(search_criteria_str, &error);
+    if (search_criteria == NULL) {
+        upnp_set_error(event, UPNP_CONTENTDIR_E_BAD_SEARCH_CRITERIA,
             _("Invalid search criteria: %s"), error);
+	free_sort_criteria(sort_criteria);
 	return false;
     }
     if (verbosity >= 4) {
 	say(4, _("Search criteria:\n"));
-	dump_search_expr(criteria->expr, 2);
+	dump_search_expr(search_criteria->expr, 2);
     }
 
     /* Check ContainerID argument */
@@ -573,6 +754,8 @@ contentdir_search(ActionEvent *event)
     if (entry == NULL) {
         upnp_set_error(event, UPNP_CONTENTDIR_E_NO_CONTAINER,
             _("No such container"));
+	free_sort_criteria(sort_criteria);
+	free_search_criteria(search_criteria);
         unlock_metadata();
 	return false;
     }
@@ -580,22 +763,26 @@ contentdir_search(ActionEvent *event)
     if (detail == NULL) {
         upnp_set_error(event, UPNP_CONTENTDIR_E_NO_CONTAINER,
             _("Not a container"));
+	free_sort_criteria(sort_criteria);
+	free_search_criteria(search_criteria);
         unlock_metadata();
 	return false;
     }
 
-    /* Do the actual searching */
-    result = strbuf_new();
-    begin_result(result);
-    match_count = search_container(criteria, entry, &index, count, filter, result);
-    end_result(result);
+    /* Do the actual searching. */
+    results = new_result_set(sort_criteria);
+    match_count = search_container(search_criteria, entry, &index, count, filter, results);
 
     /* Make response. */
-    upnp_add_response(event, "Result", strbuf_buffer(result));
+    results_str = result_set_to_string(results);
+    free_result_set(results);
+    upnp_add_response(event, "Result", results_str);
+    free(results_str);
     upnp_add_response(event, "NumberReturned", int32_str(match_count));
     upnp_add_response(event, "TotalMatches", "0");
     upnp_add_response(event, "UpdateID", uint32_str(id == 0 ? update_id : 0));
-    strbuf_free(result);
+    free_sort_criteria(sort_criteria);
+    free_search_criteria(search_criteria);
     unlock_metadata();
 
     return true;
@@ -608,11 +795,14 @@ contentdir_browse(ActionEvent *event)
     uint32_t count;
     char *flag;
     char *filter;
+    const char *error = NULL;
     int32_t id;
-    char *sort_criteria;
+    char *sort_criteria_str;
+    SortCriteria *sort_criteria;
     bool metadata;
     Entry *entry;
-    StrBuf *result;
+    TMap *results;
+    char *results_str;
 
     /* Retrieve arguments */
     index = upnp_get_ui4(event, "StartingIndex");
@@ -623,7 +813,7 @@ contentdir_browse(ActionEvent *event)
     filter = upnp_get_string(event, "Filter");
     strreplace(filter, ' ', ','); /* XXX: This is a hack. Better remove all spaces. */
     flag = upnp_get_string(event, "BrowseFlag");
-    sort_criteria = upnp_get_string(event, "SortCriteria");
+    sort_criteria_str = upnp_get_string(event, "SortCriteria");
     if (!event->status)
 	return false;
 
@@ -631,17 +821,21 @@ contentdir_browse(ActionEvent *event)
     if (count == 0)
         count = UINT32_MAX;
 
-    /* Check arguments */
-    if (strcmp(sort_criteria, "") != 0) {
+    /* Check SortCriteria. */
+    sort_criteria = parse_sort_criteria(sort_criteria_str, &error);
+    if (sort_criteria == NULL) {
         upnp_set_error(event, UPNP_CONTENTDIR_E_BAD_SORT_CRITERIA,
-            _("Sorting not supported"));
-        return false;
+            _("Invalid sort criteria: %s"), error);
+	return false;
     }
+    if (verbosity >= 4)
+	dump_sort_criteria(sort_criteria);
 
     if (strcmp(flag, "BrowseMetadata") == 0) {
 	if (index != 0) {
 	    upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
 	        _("StartingIndex must be 0 when BrowseFlag is BrowseMetaData."));
+	    free_sort_criteria(sort_criteria);
 	    return false;
 	}
 	metadata = true;
@@ -650,6 +844,7 @@ contentdir_browse(ActionEvent *event)
     } else {
 	upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
 	    _("Invalid BrowseFlag argument value (%s)"), quotearg(flag));
+	free_sort_criteria(sort_criteria);
 	return false;
     }
 
@@ -659,22 +854,22 @@ contentdir_browse(ActionEvent *event)
 	if (entry == NULL) {
             upnp_set_error(event, UPNP_CONTENTDIR_E_NO_OBJECT,
                 _("No such object"));
+	    free_sort_criteria(sort_criteria);
             unlock_metadata();
 	    return false;
 	}
 
-	result = strbuf_new();
-	begin_result(result);
-	append_entry(result, entry, filter);
-	end_result(result);
-        upnp_add_response(event, "Result", strbuf_buffer(result));
+	results = new_result_set(sort_criteria);
+	add_result_entry(results, entry, filter);
+	results_str = result_set_to_string(results);
+        free_result_set(results);
+        upnp_add_response(event, "Result", results_str);
+	free(results_str);
         upnp_add_response(event, "NumberReturned", "1");
         upnp_add_response(event, "TotalMatches", "1");
-        strbuf_free(result);
     } else {
         Entry *entry;
 	EntryDetail *detail;
-        StrBuf *result;
 	uint32_t c;
 	uint32_t end_index;
 	uint32_t result_count = 0;
@@ -683,6 +878,7 @@ contentdir_browse(ActionEvent *event)
 	if (entry == NULL) {
             upnp_set_error(event, UPNP_CONTENTDIR_E_NO_OBJECT,
                 _("No such object"));
+	    free_sort_criteria(sort_criteria);
             unlock_metadata();
 	    return false;
 	}
@@ -691,29 +887,31 @@ contentdir_browse(ActionEvent *event)
         if (detail == NULL) {
             upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
                 _("BrowseDirectChildren only possible on containers"));
+	    free_sort_criteria(sort_criteria);
             unlock_metadata();
             return false;
         }
 
-	result = strbuf_new();
-	begin_result(result);
+	results = new_result_set(sort_criteria);
 	end_index = detail->data.children.count;
 	if (count != UINT32_MAX) /* hmm.. should figure out a better way to detect if index+count overflows (>UINT32_MAX) */
 	    end_index = MIN(index+count, end_index);
         for (c = index; c < end_index; c++) {
 	    Entry *child = get_entry_by_id(detail->data.children.list[c]);
-	    append_entry(result, child, filter);
+	    add_result_entry(results, child, filter);
 	    result_count++;
         }
-	end_result(result);
 
-        upnp_add_response(event, "Result", strbuf_buffer(result));
+	results_str = result_set_to_string(results);
+        free_result_set(results);
+        upnp_add_response(event, "Result", results_str);
+	free(results_str);
         upnp_add_response(event, "NumberReturned", int32_str(result_count));
         upnp_add_response(event, "TotalMatches", int32_str(detail->data.children.count));
-        strbuf_free(result);
     }
 
     upnp_add_response(event, "UpdateID", uint32_str(id == 0 ? update_id : 0));
+    free_sort_criteria(sort_criteria);
     unlock_metadata();
 
     return event->status;
