@@ -1,6 +1,6 @@
 /* metadata.c - Management of file metadata
  *
- * Copyright (C) 2005 Oskar Liljeblad
+ * Copyright (C) 2005, 2006  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,9 @@
 #include <fcntl.h>		/* POSIX */
 #include <unistd.h>		/* POSIX */
 #include <ctype.h>		/* C89 */
+#include <inttypes.h>		/* ? */
+#include <iconv.h>		/* POSIX */
+#include "iconvme.h"		/* Gnulib */
 #include "gettext.h"		/* Gnulib/gettext */
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
@@ -50,6 +53,7 @@
 #include "getnline.h"		/* Gnulib */
 #include "strbuf.h"
 #include "strutil.h"
+#include "intutil.h"
 #include "gmediaserver.h"
 
 #define DEFAULT_ENTRIES_SIZE 		512
@@ -67,6 +71,7 @@ typedef enum {
     FILE_WMA,
     FILE_RIFF_WAVE,
     FILE_M4A,
+    FILE_OGG,
     FILE_PLS,
     FILE_M3U,
     FILE_EXTM3U,
@@ -74,12 +79,20 @@ typedef enum {
     FILE_TYPES_COUNT,
 } FileType;
 
+typedef struct _InodeList InodeList;
+
+struct _InodeList {
+    InodeList *prev;
+    ino_t node;
+};
+
 static const char *file_type_mime_types[] = {
     [FILE_MP3] 		= "audio/mpeg",
     [FILE_MP3_ID3]	= "audio/mpeg",
     [FILE_WMA] 		= "audio/x-ms-wma",
     [FILE_RIFF_WAVE] 	= "audio/x-wav",
     [FILE_M4A] 		= "audio/mp4",  
+    [FILE_OGG]		= "audio/vorbis",
     [FILE_PLS] 		= "audio/x-scpls",
     [FILE_M3U] 		= "audio/m3u",
     [FILE_EXTM3U] 	= "audio/m3u",
@@ -92,6 +105,7 @@ static const char *file_type_names[] = {
     [FILE_WMA] 		= "wma",
     [FILE_RIFF_WAVE] 	= "wav",
     [FILE_M4A] 		= "m4a",
+    [FILE_OGG]		= "ogg",
     [FILE_PLS] 		= "pls",
     [FILE_M3U] 		= "m3u",
     [FILE_EXTM3U] 	= "m3u", /* possibly extm3u in the future */
@@ -104,18 +118,23 @@ static const char *file_type_descs[] = {
     [FILE_WMA] 		= "Windows Media Audio (WMA)",
     [FILE_RIFF_WAVE] 	= "RIFF Wave",
     [FILE_M4A] 		= "MPEG v4 iTunes AAC-LC (M4A)",
+    [FILE_OGG]		= "Ogg Vorbis audio",
     [FILE_PLS]		= "PLS playlist",
     [FILE_M3U]		= "Simple M3U playlist",
     [FILE_EXTM3U]	= "Extended M3U playlist",
 };
 
-static Entry *scan_entry(const char *fullpath, const char *name, int32_t parent);
-static Entry *scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileType type);
+static Entry *scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_size, InodeList *inl);
+static Entry *scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileType type, int indent_size, InodeList *inl, ino_t node);
 
 static Entry *root_entry;
 static uint32_t entry_count = 0;
 static uint32_t entries_size = 0;
 static Entry **entries;
+#ifdef HAVE_ID3LIB
+static iconv_t iso8859_1_to_utf8;
+static iconv_t utf16_to_utf8;
+#endif
 char *file_types = DEFAULT_FILE_TYPES;
 static ithread_mutex_t metadata_mutex;
 bool tags_enabled = true;
@@ -124,7 +143,7 @@ static ID3Tag *id3;
 #endif
 
 EntryDetail *
-get_entry_detail(Entry *entry, EntryDetailType type)
+get_entry_detail(const Entry *entry, EntryDetailType type)
 {
     EntryDetail *d;
     
@@ -137,7 +156,7 @@ get_entry_detail(Entry *entry, EntryDetailType type)
 }
 
 bool
-has_entry_detail(Entry *entry, EntryDetailType type)
+has_entry_detail(const Entry *entry, EntryDetailType type)
 {
     return get_entry_detail(entry, type) != NULL;
 }
@@ -198,16 +217,14 @@ attempt_read_at(int fd, size_t size, off_t pos, uint8_t *buf, const char *fullpa
     size_t count;
 
     if (lseek(fd, pos, SEEK_SET) != pos) {
-	warn(_("%s: cannot seek: %s\n"), quotearg(fullpath), errstr);
-	close(fd); /* Ignore errors since we opened for reading */
+	warn(_("%s: cannot seek: %s\n"), quotearg(conv_filename(fullpath)), errstr);
 	return false;
     }
 
     count = full_read(fd, buf, size);
     if (count < size) {
 	if (errno != 0)
-	    warn(_("%s: cannot read: %s\n"), quotearg(fullpath), errstr);
-	close(fd); /* Ignore errors since we opened for reading */
+	    warn(_("%s: cannot read: %s\n"), quotearg(conv_filename(fullpath)), errstr);
 	return false;
     }
     return true;
@@ -222,12 +239,14 @@ check_file_content_type(const char *fullpath)
     /* XXX: Use stdio instead of fd syscalls */
     fd = open(fullpath, O_RDONLY);
     if (fd < 0) {
-	warn(_("%s: cannot open for reading: %s\n"), quotearg(fullpath), errstr);
+	warn(_("%s: cannot open for reading: %s\n"), quotearg(conv_filename(fullpath)), errstr);
 	return FILE_UNKNOWN;
     }
 
-    if (!attempt_read_at(fd, 4, 0, buf, fullpath))
+    if (!attempt_read_at(fd, 4, 0, buf, fullpath)) {
+        close(fd);
 	return FILE_UNKNOWN;
+    }
 
     /* MPEG ADTS, layer III, v1 */
     if (buf[0] == 0xFF && (buf[1] & 0xFE) == 0xFA) {
@@ -246,20 +265,38 @@ check_file_content_type(const char *fullpath)
     }
     /* RIFF (little-endian), WAVE audio */
     if (buf[0] == 'R' && buf[1] == 'I' && buf[2] == 'F' && buf[3] == 'F') {
-	if (!attempt_read_at(fd, 4, 8, buf, fullpath))
+	if (!attempt_read_at(fd, 4, 8, buf, fullpath)) {
+	    close(fd);
 	    return FILE_UNKNOWN;
+        }
 	if (buf[0] == 'W' && buf[1] == 'A' && buf[2] == 'V' && buf[3] == 'E') {
 	    close(fd); /* Ignore errors since we opened for reading */
 	    return FILE_RIFF_WAVE;
 	}
     }
 
+    /* Ogg Vorbis audio */
+    if (buf[0] == 'O' && buf[1] == 'g' && buf[2] == 'g' && buf[3] == 'S') {
+        if (!attempt_read_at(fd, 7, 28, buf+4, fullpath)) {
+            close(fd);
+            return FILE_UNKNOWN;
+        }
+        if (buf[4]==1 && buf[5]=='v' && buf[6]=='o' && buf[7]=='r' && buf[8]=='b' && buf[9]=='i' && buf[10]=='s') {
+	    close(fd); /* Ignore errors since we opened for reading */
+	    return FILE_OGG;
+        }
+    }
+
     /* ISO Media, MPEG v4 system, iTunes AAC-LC */
-    if (!attempt_read_at(fd, 4, 4, buf+4, fullpath))
+    if (!attempt_read_at(fd, 4, 4, buf+4, fullpath)) {
+        close(fd);
 	return FILE_UNKNOWN;
+    }
     if (buf[4] == 'f' && buf[5] == 't' && buf[6] == 'y' && buf[7] == 'p') {
-	if (!attempt_read_at(fd, 4, 8, buf+8, fullpath))
+	if (!attempt_read_at(fd, 4, 8, buf+8, fullpath)) {
+	    close(fd);
 	    return FILE_UNKNOWN;
+        }
 	if (buf[8] == 'M' && buf[9] == '4' && buf[10] == 'A') {
 	    close(fd); /* Ignore errors since we opened for reading */
 	    return FILE_M4A;
@@ -267,19 +304,22 @@ check_file_content_type(const char *fullpath)
     }
 
     /* Extended M3U */
-    if (memcmp(buf, "#EXTM3U", 7) == 0 && (buf[7] == '\r' || buf[7] == '\n')) {
+    if (buf[0]=='#' && buf[1]=='E' && buf[2]=='X' && buf[3]=='T' && buf[4]=='M' && buf[5]=='3' && buf[6]=='U' && (buf[7]=='\r' || buf[7]=='\n')) {
         close(fd); /* Ignore errors since we opened for reading */
         return FILE_EXTM3U;
     }
     /* Playlist (PLS) */
-    if (!attempt_read_at(fd, 4, 8, buf+8, fullpath))
+    if (!attempt_read_at(fd, 4, 8, buf+8, fullpath)) {
+        close(fd);
 	return FILE_UNKNOWN;
+    }
     if (memcmp(buf, "[playlist]", 10) == 0 && (buf[10] == '\r' || buf[10] == '\n')) {
 	close(fd); /* Ignore errors since we opened for reading */
 	return FILE_PLS;
     }
+
     /* Simple M3U */
-    if (ends_with_nocase(fullpath, ".m3u") == 0) {
+    if (ends_with_nocase(fullpath, ".m3u")) {
 	close(fd); /* Ignore errors since we opened for reading */
 	return FILE_M3U;
     }
@@ -289,23 +329,143 @@ check_file_content_type(const char *fullpath)
 }
 
 #ifdef HAVE_ID3LIB
+/* XXX: move this function to a more appropriate place, perhaps lib/iconvme? */
 static char *
-get_id3_string(ID3Tag *id3tag, ID3_FrameID frame_id)
+iconv_alloc_term(iconv_t *cd, const char *inbuf, size_t inbytes_remaining, size_t nullterm)
+{
+    char *dest;
+    char *outp;
+    /* Guess the maximum length the output string can have.  */
+    size_t outbuf_size = inbytes_remaining + nullterm;
+    size_t outbytes_remaining;
+    size_t err;
+    bool have_error = false;
+
+    /* Use a worst-case output size guess, so long as that wouldn't be
+       too large for comfort.  It's OK if the guess is wrong so long as
+       it's nonzero.  */
+    size_t approx_sqrt_SIZE_MAX = SIZE_MAX >> (sizeof (size_t) * CHAR_BIT / 2);
+    if (outbuf_size <= approx_sqrt_SIZE_MAX / MB_LEN_MAX)
+      outbuf_size *= MB_LEN_MAX;
+    outbytes_remaining = outbuf_size - nullterm;
+
+    outp = dest = xmalloc(outbuf_size);
+
+again:
+    err = iconv(cd, (char **) &inbuf, &inbytes_remaining, &outp, &outbytes_remaining);
+
+    if (err == (size_t) -1) {
+        switch (errno) {
+        case EINVAL:
+            /* Incomplete text, do not report an error */
+            break;
+        case E2BIG: {
+            size_t used = outp - dest;
+            size_t newsize = outbuf_size * 2;
+            char *newdest;
+
+            if (newsize <= outbuf_size) {
+                errno = ENOMEM;
+                have_error = true;
+                goto out;
+            }
+            newdest = xrealloc(dest, newsize);
+            dest = newdest;
+            outbuf_size = newsize;
+
+            outp = dest + used;
+            outbytes_remaining = outbuf_size - used - nullterm;
+
+            goto again;
+        }
+        case EILSEQ:
+            have_error = true;
+            break;
+        default:
+            have_error = true;
+            break;
+        }
+    }
+
+    for (; nullterm > 0; nullterm--)
+        *outp++ = '\0';
+
+out:
+    if (have_error && dest != NULL) {
+        int save_errno = errno;
+        free(dest);
+        errno = save_errno;
+        dest = NULL;
+    }
+
+    return dest;
+}
+
+static char *
+id3_frame_name(ID3_FrameID frame_id)
+{
+    switch (frame_id) {
+    case ID3FID_TITLE:
+        return _("title");
+    case ID3FID_ALBUM:
+        return _("album");
+    case ID3FID_LEADARTIST:
+        return _("artist");
+    case ID3FID_CONTENTTYPE:
+        return _("genre");
+    case ID3FID_TRACKNUM:
+        return _("track number");
+    default:
+        return _("unknown frame");
+    }
+}
+
+static char *
+get_id3_string(const char *filename, ID3Tag *id3tag, ID3_FrameID frame_id)
 {
     ID3Frame *frame;
     ID3Field *field;
-    size_t size;
-    char *buf;
 
     frame = ID3Tag_FindFrameWithID(id3tag, frame_id);
     if (frame != NULL) {
 	field = ID3Frame_GetField(frame, ID3FN_TEXT);
 	if (field != NULL) {
-	    size = ID3Field_Size(field);
-	    buf = xmalloc(size+1);
-	    ID3Field_GetASCII(field, buf, size);
-	    buf[size] = '\0';
-	    return buf;
+	    size_t insize;
+	    char *inbuf;
+	    char *outbuf = NULL;
+
+	    insize = ID3Field_Size(field);
+	    if (insize == 0)
+	        return xstrdup("");
+	    inbuf = xmalloc(insize*2);
+	    if (ID3Field_GetASCII(field, inbuf, insize) > 0) {
+                outbuf = iconv_alloc_term(iso8859_1_to_utf8, inbuf, insize, 1);
+                if (outbuf == NULL)
+                    warn(_("%s: cannot convert ID3 %s tag text: %s\n"), quotearg(conv_filename(filename)), id3_frame_name(frame_id), errstr);
+                /*outbuf = xstrndup(inbuf, insize);*/
+	    }
+	    /* insize/sizeof(unicode_t) isn't very nice here. But unfortunately it is necessary,
+	     * probably because of a bug in id3lib 3.8.3:
+	     *
+             * size_t ID3_FieldImpl::Get(unicode_t *buffer, size_t maxLength)
+             *    size_t size = this->Size();                                  [size in bytes]
+             *    length = dami::min(maxLength, size);                         [wrong!]
+             *    ::memcpy((void *)buffer, (void *)_text.data(), length * 2);  [wrong!]
+             * [..]
+             */
+	    else if (ID3Field_GetUNICODE(field, (unicode_t *) inbuf, insize/sizeof(unicode_t)) > 0) {
+                outbuf = iconv_alloc_term(utf16_to_utf8, inbuf, insize, 1);
+                if (outbuf == NULL) {
+                    warn(_("%s: cannot convert ID3 %s tag text: %s\n"), quotearg(conv_filename(filename)), id3_frame_name(frame_id), errstr);
+                } else {
+                    /* Get rid of UTF-8 BOM (see http://en.wikipedia.org/wiki/Byte_Order_Mark), */
+                    /* which is inserted by iconv if there's an UTF-16 LE/BE BOM in input text. */
+                    if (outbuf[0] == (char)0xEF && outbuf[1] == (char)0xBB && outbuf[2] == (char)0xBF)
+	                memmove(outbuf, outbuf+3, strlen(outbuf+3)+1);
+                }
+            }
+            free(inbuf);
+            return outbuf;
 	}
     }
 
@@ -314,17 +474,31 @@ get_id3_string(ID3Tag *id3tag, ID3_FrameID frame_id)
 #endif
 
 static bool
-add_playlist_child(Entry *parent, size_t *children_size, char *buf, const char *playlist_dir)
+inode_in_list(ino_t node, InodeList *inl)
+{
+    for (; inl != NULL; inl = inl->prev) {
+        if (inl->node == node)
+            return true;
+    }
+    return false;
+}
+
+static bool
+add_playlist_child(Entry *parent, size_t *children_size, char *buf, const char *playlist_dir, int indent_size, InodeList *inl)
 {
     Entry *child;
     char *child_path = buf;
+    char *child_name;
 
     if (buf[0] != '/' && strncasecmp(buf, "http://", 7) != 0)
         child_path = concat_filenames(playlist_dir, buf);
     /* XXX: using base_name here may not be entire appropriate -
      * maybe child_path is an url?
      */
-    child = scan_entry(child_path, base_name(child_path), parent->id); 
+
+    child_name = xstrdup(conv_filename(base_name(child_path)));
+    child = scan_entry(child_path, child_name, parent->id, indent_size, inl);
+    free(child_name);
     if (child != NULL) {
         EntryChildren *children;
 
@@ -343,8 +517,33 @@ add_playlist_child(Entry *parent, size_t *children_size, char *buf, const char *
     return child != NULL;
 }
 
+static bool
+fix_playlist_path(char *path)
+{
+    char *p;
+
+    /* Deny empty and absolute paths */
+    if (path[0] == '\0' || path[0] == '/' || path[0] == '\\')
+        return false;
+
+    /* Convert backslashes to slashes. Deny '..' at the same time. */
+    p = path;
+    for (; *path; path++) {
+        if (*path == '\\' || *path == '/') {
+            *path = '/';
+            if (path-p == 2 && p[0] == '.' && p[1] == '.')
+                return false;
+            p = path+1;
+        }
+    }
+    if (path-p == 2 && p[0] == '.' && p[1] == '.')
+        return false;
+
+    return true;
+}
+
 static Entry *
-scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileType type)
+scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileType type, int indent_size, InodeList *inl, ino_t node)
 {
     FILE *fh;
     size_t alloc_max;
@@ -353,17 +552,31 @@ scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileT
     ssize_t len;
     Entry *entry;
     char *playlist_dir;
-    uint32_t children_size = DEFAULT_PLAYLIST_ENTRIES; /* just some basic initial size */
+    size_t children_size = DEFAULT_PLAYLIST_ENTRIES; /* just some basic initial size */
     int ln;
     int bad = 0;
+    char *indent;
+    InodeList inl_new;
 
-    say(4, _("Scanning playlist %s\n"), quote(fullpath));
-    fh = fopen(fullpath, "r");
-    if (fh == NULL) {
-        warn(_("%s: cannot open file for reading: %s\n"), quotearg(fullpath), errstr);
+    indent = xstrdupn(" ", indent_size*2);
+
+    if (inode_in_list(node, inl)) {
+        warn(_("%s: recursive loop detected - ignoring\n"), quotearg(conv_filename(fullpath)));
+        free(indent);
         return NULL;
     }
 
+    say(4, _("%sScanning playlist %s\n"), indent, quote(conv_filename(fullpath)));
+
+    fh = fopen(fullpath, "r");
+    if (fh == NULL) {
+        warn(_("%s: cannot open file for reading: %s\n"), quotearg(conv_filename(fullpath)), errstr);
+        free(indent);
+        return NULL;
+    }
+
+    inl_new.prev = inl;
+    inl_new.node = node;
     playlist_dir = dir_name(fullpath);
     entry = make_entry(name, parent, true);
 
@@ -397,49 +610,55 @@ scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileT
                     int c;
 
                     for (c = 4; c < len && buf[c] >= '0' && buf[c] <= '9'; c++);
-                    if (c == 4 || c >= len || buf[c] != '=') {
-                        warn(_("%s: invalid line %d\n"), quotearg(fullpath), ln);
+                    if (c == 4 || c >= len || buf[c] != '=' || !fix_playlist_path(buf+c+1)) {
+                        warn(_("%s: invalid line %d\n"), quotearg(conv_filename(fullpath)), ln);
                         break;
                     }
                     /* Ignore the numbering of files in PLS playlists. */
-                    if (!add_playlist_child(entry, &children_size, buf+c+1, playlist_dir)) {
+                    if (!add_playlist_child(entry, &children_size, buf+c+1, playlist_dir, indent_size+1, &inl_new)) {
                         bad++;
                         if (bad > MAX_INVALID_PLS_FILES) {
                             /* For now, just break out of the loop. */
-                            warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(fullpath), MAX_INVALID_PLS_FILES);
+                            warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(conv_filename(fullpath)), MAX_INVALID_PLS_FILES);
                             break;
                         }
                     }
                 } else {
-                    warn(_("%s: invalid line %d\n"), quotearg(fullpath), ln);
+                    warn(_("%s: invalid line %d\n"), quotearg(conv_filename(fullpath)), ln);
                     bad++;
                     if (bad > MAX_INVALID_PLS_LINES) {
                         /* For now, just break out of the loop. */
-                        warn(_("%s: too many invalid lines (max allowed %d)\n"), quotearg(fullpath), MAX_INVALID_PLS_LINES);
+                        warn(_("%s: too many invalid lines (max allowed %d)\n"), quotearg(conv_filename(fullpath)), MAX_INVALID_PLS_LINES);
                         break;
                     }
                 }
             } else if (type == FILE_EXTM3U) {
                 if (len >= 8 && strncmp(buf, "#EXTINF:", 8) == 0) {
                     /* No operation - we're not interested in EXTINF */
-                } else if (len > 0) {
-                    if (!add_playlist_child(entry, &children_size, buf, playlist_dir)) {
+                } else if (len == 0 || !fix_playlist_path(buf)) {
+                    warn(_("%s: invalid line %d\n"), quotearg(conv_filename(fullpath)), ln);
+                    bad++;
+                } else {
+                    if (!add_playlist_child(entry, &children_size, buf, playlist_dir, indent_size+1, &inl_new))
                         bad++;
-                        if (bad > MAX_INVALID_EXTM3U_FILES) {
-                            /* For now, just break out of the loop. */
-                            warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(fullpath), MAX_INVALID_EXTM3U_FILES);
-                            break;
-                        }
-                    }
+                }
+                if (bad > MAX_INVALID_EXTM3U_FILES) {
+                    /* For now, just break out of the loop. */
+                    warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(conv_filename(fullpath)), MAX_INVALID_EXTM3U_FILES);
+                    break;
                 }
             } else if (type == FILE_M3U) {
-                if (!add_playlist_child(entry, &children_size, buf, playlist_dir)) {
+                if (len == 0 || !fix_playlist_path(buf)) {
+                    warn(_("%s: invalid line %d\n"), quotearg(conv_filename(fullpath)), ln);
                     bad++;
-                    if (bad > MAX_INVALID_M3U_FILES) {
-                        /* For now, just break out of the loop. */
-                        warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(fullpath), MAX_INVALID_M3U_FILES);
-                        break;
-                    }
+                } else {
+                    if (!add_playlist_child(entry, &children_size, buf, playlist_dir, indent_size+1, &inl_new))
+                        bad++;
+                }
+                if (bad > MAX_INVALID_M3U_FILES) {
+                    /* For now, just break out of the loop. */
+                    warn(_("%s: too many invalid files (max allowed %d)\n"), quotearg(conv_filename(fullpath)), MAX_INVALID_M3U_FILES);
+                    break;
                 }
             }
         }
@@ -449,33 +668,39 @@ scan_playlist_file(const char *fullpath, const char *name, int32_t parent, FileT
      * entries.
      */
     if (ferror(fh))
-        warn(_("%s: cannot read from file: %s\n"), quotearg(fullpath), errstr);
+        warn(_("%s: cannot read from file: %s\n"), quotearg(conv_filename(fullpath)), errstr);
 
     fclose(fh); /* Ignore errors because we opened for reading */
     free(buf);
     free(playlist_dir);
+    free(indent);
     return entry;
 }
     
 static Entry *
-scan_entry(const char *fullpath, const char *name, int32_t parent)
+scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_size, InodeList *inl)
 {
     struct stat sb;
+    char *indent;
+
+    indent = xstrdupn(" ", indent_size*2);
 
     if (strncasecmp(fullpath, "http://", 7) == 0) {
         Entry *entry;
         EntryDetail *detail;
 
-        say(4, _("Adding URL %s as %s\n"), quote_n(0, fullpath), quote_n(1, name));
+        say(4, _("%sAdding URL %s as %s\n"), indent, quote_n(0, conv_filename(fullpath)), quote_n(1, name));
         entry = make_entry(name, parent, false);
         detail = add_entry_detail(entry, DETAIL_URL);
         detail->data.url.url = xstrdup(fullpath);
 
+	free(indent);
         return entry;
     }
 
     if (stat(fullpath, &sb) < 0) {
-        warn(_("%s: cannot stat: %s\n"), quotearg(fullpath), errstr);
+        warn(_("%s: cannot stat: %s\n"), quotearg(conv_filename(fullpath)), errstr);
+	free(indent);
         return NULL;
     }
 
@@ -487,14 +712,24 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
         struct dirent **dirents;
         int dirent_count;
         int c;
+        InodeList inl_new;
+
+	say(4, _("%sScanning directory %s\n"), indent, quote(conv_filename(fullpath)));
+	if (inode_in_list(sb.st_ino, inl)) {
+	    warn(_("%s: recursive loop detected - ignoring\n"), quotearg(conv_filename(fullpath)));
+	    free(indent);
+	    return NULL;
+	}
 
-	say(4, _("Scanning directory %s\n"), quote(fullpath));
         dirent_count = scandir(fullpath, &dirents, NULL, alphasort);
         if (dirent_count < 0) {
-            warn(_("%s: cannot scan directory: %s\n"), quotearg(fullpath), errstr);
+            warn(_("%s: cannot scan directory: %s\n"), quotearg(conv_filename(fullpath)), errstr);
+	    free(indent);
             return NULL;
         }
 
+        inl_new.prev = inl;
+        inl_new.node = sb.st_ino;
         entry = make_entry(name, parent, true);
 	children = xmalloc(sizeof(int32_t) * dirent_count);
 
@@ -503,9 +738,12 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
             if (strcmp(dirents[c]->d_name, ".") != 0 && strcmp(dirents[c]->d_name, "..") != 0) {
 		Entry *child;
 		char *child_path;
+		char *child_name;
 
 		child_path = concat_filenames(fullpath, dirents[c]->d_name);
-		child = scan_entry(child_path, dirents[c]->d_name, entry->id);
+		child_name = xstrdup(conv_filename(dirents[c]->d_name));
+		child = scan_entry(child_path, child_name, entry->id, indent_size+1, &inl_new);
+		free(child_name);
 		if (child != NULL)
 		    children[child_count++] = child->id;
 		free(child_path);
@@ -517,6 +755,7 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
 	detail->data.children.list = xmemdup(children, sizeof(int32_t) * child_count);
 	free(children);
 
+	free(indent);
         return entry;
     }
 
@@ -526,102 +765,199 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
 	FileType type;
 
         if (parent == -1) {
-            warn(_("%s: root must be a directory\n"), quotearg(fullpath));
+            warn(_("%s: root must be a directory\n"), quotearg(conv_filename(fullpath)));
+	    free(indent);
             return NULL;
         }
-        say(4, _("Checking content type of file %s\n"), quote(fullpath));
+
+        say(4, _("%sFound regular file %s\n"), indent, quote(name));
 	type = check_file_content_type(fullpath);
 	if (type == FILE_UNKNOWN) {
-	    say(4, _("Matched no type for %s\n"), quote(fullpath));
+	    say(4, _("%s  Matched no type\n"), indent);
             /*say(4, _("%s: skipping file - unrecognized content type\n"), quotearg(fullpath));*/
         } else if (type == FILE_M3U) {
-	    say(4, _("Assuming type %s for %s\n"), file_type_descs[type], quote(fullpath));
+	    say(4, _("%s  Assuming type %s - %s\n"), indent, file_type_descs[type], file_type_mime_types[type]);
         } else {
-	    say(4, _("Matched type %s for %s\n"), file_type_descs[type], quote(fullpath));
+	    say(4, _("%s  Matched type %s - %s\n"), indent, file_type_descs[type], file_type_mime_types[type]);
         }
-	if (!string_in_csv(file_types, ',', file_type_names[type]))
+	if (!string_in_csv(file_types, ',', file_type_names[type])) {
+	    say(4, _("%s  Skipping (file type excluded)\n"), indent);
+	    free(indent);
 	    return NULL;
+	}
 
-        if (type == FILE_PLS || type == FILE_M3U || type == FILE_EXTM3U)
-            return scan_playlist_file(fullpath, name, parent, type);
+        if (type == FILE_PLS || type == FILE_M3U || type == FILE_EXTM3U) {
+            free(indent);
+	    return scan_playlist_file(fullpath, name, parent, type, indent_size+1, inl, sb.st_ino);
+	}
 
-        say(4, _("Adding file %s as %s\n"), quote_n(0, fullpath), quote_n(1, name));
+        /*say(4, _("%s  Adding as %s\n"), indent, quote(name));*/
         entry = make_entry(name, parent, false);
 
         detail = add_entry_detail(entry, DETAIL_FILE);
         detail->data.file.size = sb.st_size;
+	detail->data.file.mode = sb.st_mode;
         detail->data.file.mtime = sb.st_mtime;
         detail->data.file.filename = xstrdup(fullpath);
 	detail->data.file.mime_type = file_type_mime_types[type];
+	/* XXX: dump size, mtime, ?? */
 
 	if (tags_enabled && (type == FILE_MP3 || type == FILE_MP3_ID3)) {
-#ifdef HAVE_ID3LIB
-	    /*char *title;*/
-	
-	    ID3Tag_Clear(id3);
-	    ID3Tag_Link(id3, fullpath);
+	    detail = NULL;
 
-            if (ID3Tag_NumFrames(id3) != 0) {
-                detail = add_entry_detail(entry, DETAIL_TAG);
-                /*title = get_id3_string(id3, ID3FID_TITLE);
-                if (title != NULL && strcmp(title, "") != 0) {
-                    free(entry->name);
-                    entry->name = title;
-                } else {
-                    free(title);
-                }*/
-                detail->data.tag.title = get_id3_string(id3, ID3FID_TITLE);
-                detail->data.tag.album = get_id3_string(id3, ID3FID_ALBUM);
-                detail->data.tag.artist = get_id3_string(id3, ID3FID_LEADARTIST);
-                detail->data.tag.genre = get_id3_string(id3, ID3FID_CONTENTTYPE);
-	    }
-#endif
 #ifdef HAVE_TAGLIB
-            {
+            if (detail == NULL) {
                 TagLib_File *tl_file;
+                TagLib_File_Type tl_type;
 
-		/* FIXME: change TagLib_File_MPEG when OGG support added */
-                tl_file = taglib_file_new_type(fullpath, TagLib_File_MPEG); 
+                switch (type) {
+                case FILE_OGG:
+                    tl_type = TagLib_File_OggVorbis;
+                    break;
+                default:
+                    tl_type = TagLib_File_MPEG;
+                    break;
+                }
+
+                tl_file = taglib_file_new_type(fullpath, tl_type); 
                 if (tl_file != NULL) {
                     TagLib_Tag *tl_tag;
+                    const TagLib_AudioProperties *tl_props;
+		    char *str;
+                    unsigned int track;
 
                     tl_tag = taglib_file_tag(tl_file);
                     detail = add_entry_detail(entry, DETAIL_TAG);
-                    detail->data.tag.title = xstrdup(taglib_tag_title(tl_tag));
+		    str = taglib_tag_title(tl_tag);
+		    detail->data.tag.title = (*str == '\0' ? NULL : xstrdup(str));
+		    str = taglib_tag_album(tl_tag);
+                    detail->data.tag.album = (*str == '\0' ? NULL : xstrdup(str));
+		    str = taglib_tag_artist(tl_tag);
+                    detail->data.tag.artist = (*str == '\0' ? NULL : xstrdup(str));
+		    str = taglib_tag_genre(tl_tag);
+                    detail->data.tag.genre = (*str == '\0' ? NULL : xstrdup(str));
+                    track = taglib_tag_track(tl_tag);
+                    detail->data.tag.track_no = track > 0 ? track : -1;
+                    tl_props = taglib_file_audioproperties(tl_file);
+                    detail->data.tag.duration = taglib_audioproperties_length(tl_props);
 
-                    detail->data.tag.album = xstrdup(taglib_tag_album(tl_tag));
-                    detail->data.tag.artist = xstrdup(taglib_tag_artist(tl_tag));
-                    detail->data.tag.genre = xstrdup(taglib_tag_genre(tl_tag));
-                    taglib_tag_free_strings();
+                    /*taglib_tag_free_strings();*/
                     taglib_file_free(tl_file);
                 }
             }
 #endif
+#ifdef HAVE_ID3LIB
+            if (detail == NULL) {
+                /*char *title;*/
+
+                ID3Tag_Clear(id3);
+                ID3Tag_Link(id3, fullpath);
+
+                if (ID3Tag_NumFrames(id3) != 0) {
+                    char *track;
+
+                    detail = add_entry_detail(entry, DETAIL_TAG);
+                    /*title = get_id3_string(fullpath, id3, ID3FID_TITLE);
+                    if (title != NULL && strcmp(title, "") != 0) {
+                        free(entry->name);
+                        entry->name = title;
+                    } else {
+                        free(title);
+                    }*/
+                    detail->data.tag.title = get_id3_string(fullpath, id3, ID3FID_TITLE);
+                    detail->data.tag.album = get_id3_string(fullpath, id3, ID3FID_ALBUM);
+                    detail->data.tag.artist = get_id3_string(fullpath, id3, ID3FID_LEADARTIST);
+                    detail->data.tag.genre = get_id3_string(fullpath, id3, ID3FID_CONTENTTYPE);
+                    track = get_id3_string(fullpath, id3, ID3FID_TRACKNUM);
+                    if (track == NULL || !parse_int32(track, &detail->data.tag.track_no))
+                        detail->data.tag.track_no = -1;
+
+                    /* get_id3_string(fullpath, id3, ID3FID_TIME); */
+                    detail->data.tag.duration = -1;
+                }
+            }
+#endif
+	    if (detail != NULL) {
+		if (detail->data.tag.title != NULL)
+		    say(4, "%s  %s: %s\n", indent, _("Title"), quotearg(detail->data.tag.title));
+		if (detail->data.tag.artist != NULL)
+		    say(4, "%s  %s: %s\n", indent, _("Artist"), quotearg(detail->data.tag.artist));
+		if (detail->data.tag.album != NULL)
+		    say(4, "%s  %s: %s\n", indent, _("Album"), quotearg(detail->data.tag.album));
+		if (detail->data.tag.genre != NULL)
+		    say(4, "%s  %s: %s\n", indent, _("Genre"), quotearg(detail->data.tag.genre));
+		if (detail->data.tag.track_no != -1)
+		    say(4, "%s  %s: %" PRId32 "\n", indent, _("Track number"), detail->data.tag.track_no);
+                if (detail->data.tag.duration != -1)
+                    say(4, "%s  %s: %u:%02u:%02u.%02u\n", indent, _("Duration"), SPLIT_DURATION(detail->data.tag.duration));
+	    }
         }
 
+	free(indent);
         return entry;
     }
 
-    warn(_("%s: skipping file - unsupported file type\n"), quotearg(fullpath));
+    if (S_ISBLK(sb.st_mode) || S_ISCHR(sb.st_mode) || S_ISFIFO(sb.st_mode)) {
+	Entry *entry;
+	EntryDetail *detail;
+
+        if (parent == -1) {
+            warn(_("%s: root must be a directory\n"), quotearg(conv_filename(fullpath)));
+	    free(indent);
+            return NULL;
+        }
+        say(4, _("%sFound device/FIFO file %s\n"), indent, quote(name));
+
+        entry = make_entry(name, parent, false);
+        detail = add_entry_detail(entry, DETAIL_FILE);
+        detail->data.file.size = sb.st_size;
+	detail->data.file.mode = sb.st_mode;
+        detail->data.file.mtime = sb.st_mtime;
+        detail->data.file.filename = xstrdup(fullpath);
+	detail->data.file.mime_type = file_type_mime_types[FILE_MP3]; /* FIXME: unknown type. */
+
+	free(indent);
+	return entry;
+    }
+
+    warn(_("%s: skipping file - unsupported file type\n"), quotearg(conv_filename(fullpath)));
+    free(indent);
     return NULL;
 }
 
-void
+bool
 init_metadata(void)
 {
+#ifdef HAVE_ID3LIB
+    if (tags_enabled) {
+        iso8859_1_to_utf8 = iconv_open("UTF-8", "ISO8859-1");
+        if (iso8859_1_to_utf8 == (iconv_t) -1) {
+             warn(_("cannot create character set convertor from %s to %s\n"), "ISO8859-1", "UTF-8");
+             return false;
+        }
+        /* ID3 <= v2.3 uses UTF-16, because the BOM is included. Or to be more correct,
+         * it uses UCS-2, but since that is a subset of UTF-16 and ID3 >= v2.4 uses UTF-16
+         * we use that in this convertor instead.
+         */
+        utf16_to_utf8 = iconv_open("UTF-8", "UTF-16");
+        if (utf16_to_utf8 == (iconv_t) -1) {
+             warn(_("cannot create character set convertor from %s to %s\n"), "UTF-16", "UTF-8");
+             return false;
+        }
+	id3 = ID3Tag_New();
+    }
+#endif
+
     ithread_mutex_init(&metadata_mutex, NULL);
 
     entries_size = DEFAULT_ENTRIES_SIZE;
     entries = xmalloc(sizeof(Entry *) * entries_size);
 
-#ifdef HAVE_ID3LIB
-    if (tags_enabled)
-	id3 = ID3Tag_New();
-#endif
+    return true;
 }
 
 bool
-scan_entries(char **pathv, int pathc)
+scan_entries(char **pathv, int pathc, int indent_size)
 {
     if (pathc > 1) {
 	int32_t *children;
@@ -637,13 +973,16 @@ scan_entries(char **pathv, int pathc)
 
 	    name = base_name(pathv[c]);
 	    if (*name == '\0' || *name == '/') {
-	        name = xstrdup(pathv[c]);
+	        name = xstrdup(conv_filename(pathv[c]));
             } else {
+                char *tmp;
 	        for (d = strlen(name)-1; name[d] == '/'; d--);
-	        name = xstrndup(name, d+1);
+	        tmp = xstrndup(name, d+1);
+	        name = xstrdup(conv_filename(tmp));
+	        free(tmp);
             }
 
-	    entry = scan_entry(pathv[c], name, -1);
+	    entry = scan_entry(pathv[c], name, -1, indent_size, NULL);
 	    children[c] = entry->id;
 	    free(name);
 	}
@@ -651,7 +990,7 @@ scan_entries(char **pathv, int pathc)
 	detail->data.children.count = pathc;
 	detail->data.children.list = children;
     } else {
-	root_entry = scan_entry(pathv[0], ROOT_ENTRY_NAME, -1);
+	root_entry = scan_entry(pathv[0], ROOT_ENTRY_NAME, -1, indent_size, NULL);
     }
 
 
@@ -697,8 +1036,11 @@ finish_metadata(void)
 {
     clear_entries();
 #ifdef HAVE_ID3LIB
-    if (tags_enabled)
+    if (tags_enabled) {
 	ID3Tag_Delete(id3);
+	iconv_close(iso8859_1_to_utf8); /* ignore errors (only EINVAL) */
+	iconv_close(utf16_to_utf8); /* ignore errors (only EINVAL) */
+    }
 #endif
     free(entries);
     entries_size = 0;
