@@ -1,6 +1,6 @@
 /* upnp.c - Routines dealing interfacing with the upnp library
  *
- * Copyright (C) 2005  Oskar Liljeblad
+ * Copyright (C) 2005, 2006  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <assert.h>		/* C89 */
 #include <netinet/in.h>		/* ?; inet_ntoa */
 #include <arpa/inet.h>		/* ?; inet_ntoa */
+#include <inttypes.h>		/* POSIX */
 #include "gettext.h"            /* Gnulib/gettext */
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
@@ -32,13 +33,22 @@
 #include "xvasprintf.h"		/* Gnulib */
 #include "xgethostname.h"	/* Gnulib */
 #include "xalloc.h"		/* Gnulib */
+#include <uuid/uuid.h>
 #include "intutil.h"
 #include "gmediaserver.h"
 #include "schemas/MediaServer.h"
 
+/* libupnp 1.2.x has SSDP_PAUSE, but not 1.3.x
+ * Perhaps this kludge should be moved somewhere else.
+ */
+#ifndef SSDP_PAUSE
+#define SSDP_PAUSE 100
+#endif
+
 char *friendly_name = NULL;
 int ssdp_expire_time = SSDP_PAUSE;
 static UpnpDevice_Handle device;
+static char device_udn[42]; /* uuid:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx + null-byte */
 
 static Service services[] = {
   {
@@ -194,8 +204,7 @@ notify_change(const char *service_id, ServiceVariable *variable)
     int res;
 
     say(3, _("Variable %s has changed value to %s.\n"), variable->name, quotearg(variable->value));
-
-    res = UpnpNotify(device, DEVICE_UDN, service_id, variable_names, variable_values, 1);
+    res = UpnpNotify(device, device_udn, service_id, variable_names, variable_values, 1);
     if (res != UPNP_E_SUCCESS)
         say(1, _("Cannot notify variable update - %s\n"), upnp_errmsg(res));
 }
@@ -213,7 +222,7 @@ handle_subscription_request(struct Upnp_Subscription_Request *request)
     say(3, _("Event service ID: %s\n"), quotearg(request->ServiceId));
     say(3, _("Event request SID: %s\n"), quotearg(request->Sid));
 
-    if (strcmp(request->UDN, DEVICE_UDN) != 0) {
+    if (strcmp(request->UDN, device_udn) != 0) {
         say(1, _("Discarding event - event device UDN (%s) not recognized\n"), quotearg(request->UDN));
         return;
     }
@@ -263,7 +272,7 @@ handle_get_var_request(struct Upnp_State_Var_Request *request)
     say(3, _("Event variable name: %s\n"), quotearg(request->StateVarName));
     say(3, _("Event source: %s\n"), inet_ntoa(request->CtrlPtIPAddr));
 
-    if (strcmp(request->DevUDN, DEVICE_UDN) != 0) {
+    if (strcmp(request->DevUDN, device_udn) != 0) {
         say(1, _("Discarding event - event device UDN (%s) not recognized\n"), quotearg(request->DevUDN));
         strcpy(request->ErrStr, _("Unrecognized device"));
         request->CurrentVal = NULL;
@@ -305,7 +314,7 @@ handle_action_request(struct Upnp_Action_Request *request)
     say(3, _("Event source: %s\n"), inet_ntoa(request->CtrlPtIPAddr));
     say_document(4, _("Event action request:\n"), request->ActionRequest);
 
-    if (strcmp(request->DevUDN, DEVICE_UDN) != 0) {
+    if (strcmp(request->DevUDN, device_udn) != 0) {
         say(1, _("Discarding event - event device UDN (%s) not recognized\n"), quotearg(request->DevUDN));
         strcpy(request->ErrStr, _("Unrecognized device"));
         request->ActionResult = NULL;
@@ -481,6 +490,7 @@ void
 init_upnp(const char *listenip, uint16_t listenport)
 {
     int res;
+    uuid_t device_uuid;
     char *mediaserver_desc;
 
     if (listenip != NULL)
@@ -507,11 +517,17 @@ init_upnp(const char *listenip, uint16_t listenport)
     if (res != UPNP_E_SUCCESS)
         die(_("cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
 
+    say(3, _("Generating device UDN (UUID)...\n"));
+    uuid_generate(device_uuid);
+    sprintf(device_udn, "uuid:");
+    uuid_unparse_lower(device_uuid, device_udn+strlen(device_udn));
+    say(4, _("  UDN: %s\n"), device_udn);
+
     say(3, _("Registering UPnP root device...\n"));
 
     if (friendly_name == NULL)
 	friendly_name = xasprintf(_("GMediaServer on %s"), xgethostname());
-    mediaserver_desc = xasprintf(MEDIASERVER_DESC, friendly_name);
+    mediaserver_desc = xasprintf(MEDIASERVER_DESC, device_udn, friendly_name);
     free(friendly_name);
     res = UpnpRegisterRootDevice2(UPNPREG_BUF_DESC, mediaserver_desc,
               strlen(mediaserver_desc), 1, device_callback_event_handler, NULL, &device);
